############################################################
#
# Function definitions
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
#
############################################################

declaration template pro_declaration_functions_general;

#
# Return either current value for a path if it exists and
# is defined or the given default value.
############################################################
define function default = {
   #
   # Check cardinality.
   if (argc != 1) error("usage: \"/full/path\" = default(value)");
   #
   v = argv[0];
   if (exists(self) && is_defined(self)) v = self;
   return(v);
   }; 


#
# Push a value onto the end of a list. If the list doesn't
# exist or isn't defined a new list is created.
#
# push <value>
#
############################################################
define function push = {
   #
   # Check cardinality.
   if (argc != 1) error("usage: \"/full/path\" = push(value)");
   #
   # If the path doesn't exist yet, add the empty list.
   if (exists(self) && is_list(self)) {
      v = self;
      v[length(v)] = argv[0];
      return(v);
      };
   #
   # If doesn't exist or is undefined, make new list.
   if (!exists(self) || !is_defined(self)) {
      v = list(argv[0]);
      return(v);
      };
   #
   # If we got here, self exists and is defined but isn't a list.
   error("push can only be applied to a list");
   }; 


#
# Push a value into a nlist. If the nlist doesn't
# exist or isn't defined a new list is created.
# I the key already exists in that list an error
# will be dropped
#
# npush <key> <value>
#
############################################################

define function npush = {
   # general testing
   if ( argc != 2 ) {
      error("usage: \"full/path\" = npush(key , value)");
      };
   
   # If the path does exist, add the new entry if it does not exist yet.
   if (exists(self) && is_nlist(self)) {
      v = self;
      if (!exists(v[argv[0]]) ) {
         v[argv[0]] = argv[1];
         return(v);
         };
      error("entry " + argv[0] + " already exists");
      };

   # If the path does not exist or is undefined, make new list.
   if (!exists(self) || !is_defined(self)) {
      v = nlist(argv[0], argv[1]);
      return(v);
      };

   # If we got here, self exists and is defined but is not a nlist.
   error("npush can only be applied to a nlist");
   };


#
# Push a value onto the end of a list if the given expression
# is true. If the boolean is false, then either "self"
# if self exists) or the empty list is returned.
#
# push_if <boolean> <value>
#
############################################################
define function push_if = {
   # Check cardinality and first argument.
   if (argc != 2 || !is_boolean(argv[0]))
     error("usage: \"/full/path\" = push(boolean, value)");

   # Standard push if boolean is true.
   if (argv[0]) {
      # Just do a standard push.
      return(push(argv[1]));
   } else {
   # Return either self or an empty list.
     if (is_defined(self)) {
        return(self);
     } else {
        return(list());
     };
   };

};


#
# TODO remove this very specialized function
#
# Check that item with a given name exists in an item list
#
# mab: an "item list" is a list of nlists, where there is a "name" field in each
# of the nlists
#
# named_item_exists <name> <item list name>
#
###################################################################

define function named_item_exists = {
    name = argv[0];
    item_list_name = argv[1];
    item_list = value(item_list_name);
    i = first(item_list,t,curr_item);
    while(i) {
	if(name == curr_item["name"]) return(true);
	i = next(item_list,t,curr_item);
    };
    debug("Item " + name + " does not exist in " + item_list_name);
    return(false);
};



# Calculate sum of a given field values for a given item list
#
# sum_item_list <item list name> <field name>
#
# Martin Bock:
# currently commented out since not needed, however this one might come in handy
#
########################################################################

#define function sum_item_list = {
#    item_list_name = argv[0];
#    item_list = value(item_list_name);
#    field_name = argv[1];
#    sum = 0;
#    i = first(item_list,t,curr_item);
#    while(i) {
#	sum = sum + to_long(curr_item[field_name]);
#	i = next(item_list,t,curr_item);
#    };
#    return(sum);
#};



# Automatically fill "repository" field for package list
#
# resolve_pkg_rep <repository list>
#
########################################################

define function resolve_pkg_rep = {
    error=0;
    errorstr="";
    pkg_list = self;
    rep_list = argv[0];
    debug("Assigning repositories to packages...");
    p = first(pkg_list,name,pkg_list_name);
    while(p) {
      l = first(pkg_list_name,version,pkg_list_name_version);
      while(l) {
	  id = escape(unescape(name) + "-" + unescape(version) + "-" + pkg_list_name_version["arch"]);
	  if(exists(pkg_list_name_version["repository"])) {
	        rep_mask = pkg_list_name_version["repository"];
		delete(pkg_list_name_version["repository"]);
	  } else {
		rep_mask = "";
	  };
	  k = first(rep_list,t,curr_rep);
	  while(k) {
	       if(match(curr_rep["name"],rep_mask) && exists(curr_rep["contents"][id])) {
	          debug("Package " + unescape(name) + " - assigned repository " + curr_rep["name"]);
	          pkg_list_name_version["repository"] = curr_rep["name"];
	          k = false;
	       } else {
	          k = next(rep_list,t,curr_rep);
	       };
	  };
	  if(! exists(pkg_list[name][version]["repository"])) {
	     errorstr = errorstr+"\n  name: "+unescape(name)+" version: "+unescape(version)+"";
	     error=error+1;
	     # error("cannot find repository for package " + unescape(name) + " version "+unescape(version));
	  };
	  l = next(pkg_list_name,version,pkg_list_name_version);
      };
      p = next(pkg_list,name,pkg_list_name);
    };

    if (error >0) {
      error ("cannot find any repository holding the following package(s): "+errorstr+"\n");
    };
    return(pkg_list);
};

# Remove unused repositories
#
# purge_rep_list <package list>
#
# really needed function, or optional???
#
############################

define function purge_rep_list = {
    rep_list = self;
    pkg_list = argv[0];
    used_rep_names = nlist();
    debug("Gathering repository usage statistics...");
    p = first(pkg_list,name,pkg_list_name);
    while(p) {
        l = first(pkg_list_name,version,pkg_list_name_version);
        while(l) {
        	rep = escape(pkg_list_name_version["repository"]);
		if(exists(used_rep_names[rep])) {
		    used_rep_names[rep] = used_rep_names[rep] + 1;
		} else {
		    used_rep_names[rep] = 1;
		};
		l = next(pkg_list_name,version,pkg_list_name_version);
        };
        p = next(pkg_list,name,pkg_list_name);
    };
    k = length(rep_list) - 1;
    while(k >= 0) {
	if(exists(used_rep_names[escape(rep_list[k]["name"])])) {
	    debug("Purging repository " + rep_list[k]["name"] + " (used " + to_string(used_rep_names[escape(rep_list[k]["name"])]) + " times)");
	    delete(rep_list[k]["contents"]);
	} else {
	    debug("Removing unused repository " + rep_list[k]["name"]);
	    delete(rep_list[k]);
	};
	k = k - 1;
    };
    return(rep_list);
};



# Remove package from list
#
# pkg_del <name> [version]
#
# if version is not specified (no argument provided),
# then ALL existing versions are removed from the profile.
#
########################################

define variable DEF = "";

define function pkg_del = {
    pkg_list = self;
    name = argv[0];
    debug("Removing package " + name);
    e_name = escape(name);
    if(argc > 1 && argv[1] != DEF) {
	e_version = escape(argv[1]);
    } else {
	debug("    Version not specified, deleting all versions");
	delete(pkg_list[e_name]);
	return(pkg_list);
    };
    delete(pkg_list[e_name][e_version]);
    return(pkg_list);
};

# Add package to the package list
#
# pkg_add <name> [version] [arch] [multi] [flags] [repository]
#
# examples:
#
# add emacs-19.34.i386 to the profile
# "/software/packages"=pkg_add("emacs","19.34","i386");
#
# add the default version of emacs to the profile
# "/software/packages"=pkg_add("emacs");
#
# add two kernel versions to the profile
# "/software/packages"=pkg_add("kernel","2.4.1","i386","1");
# "/software/packages"=pkg_add("kernel","2.4.2","i386","1");
#
# use repository "CERN_CC" for the package CERNLIB
# "/software/packages"=pkg_add("cernlib","3.0.1","i386",DEF,DEF,"CERN_CC")
#
#
# Only the <name> is mandatory.
#
# If [version] or [arch] are not defined, their value is deduced
# from the global variable package_default. (If no entry in 
# package_default is found, an error is produced)
#
# the optional 'multi' argument indicates if this package
# version can live with other package versions in this profile.
# (eg. when using multiple versions of a package, like the
# Linux Kernel RPMs or the ASIS gcc-alt RPMs)
# if this multi argument is set (ie. defined and set to a value
# different to DEF), then multiple versions are allowed for this
# product.
# If the multi value is not set, the pkg_add function will fail
# if a previous version is found in the profile.
#
# the flags and repository fields are optional.
#
# the repository field can be used to force the usage of a
# specific repository.
#
# the flag field will be used for future extensions.
#
########################################
define function pkg_add = {
    pkg_list = self;
    name = argv[0];
    u_name = '_'+name;
    if (argc > 1 && argv[1] != DEF) {
      version = argv[1];
    } else {
      if (exists(package_default[u_name][0])) {
	version = package_default[u_name][0];
      } else {
        error("no default package version defined for package: "+name);
      };
    };
    if (argc > 2 && argv[2] != DEF) {
      arch = argv[2];
    } else {
      if (exists(package_default[u_name][1])) {
        arch = package_default[u_name][1];
      } else {
        error("no default package architecture defined for package: "+name);
      };
    };
    debug("Adding package " + name + " version " + version + " arch " + arch);
    e_name = escape(name);
    e_version = escape(version);
    if (argc > 3) {
        multi = argv[3];
    } else {
        multi = DEF;
    };
    if ((multi == DEF) && (exists(pkg_list[e_name]))) {
          error(" pkg_add: package "+name+" found twice in profile, but no multiversion specified"); # ??? list all unescaped versions in pkg_list[name]
    };
    if(argc > 4) {
        flags = argv[4];
    } else {
        flags = DEF;
    };
    if(argc > 5) {
        rep = argv[5];
    } else {
        rep = DEF;
    };
    pkg_list[e_name][e_version] = nlist("arch",arch);
    if(is_nlist(flags)) {
        pkg_list[e_name][e_version]["flags"] = flags;
    };
    if(rep != DEF) {
        pkg_list[e_name][e_version]["repository"] = rep;
    };
    return(pkg_list);
};

# Replace package in the list
#
# pkg_repl <name> <new version> <arch> [old version] [flags] [repository]
#
# if old version is not specified, all versions are replaced.
#
# if no version existed in the profile, the new version
# is just added.
#
# See pkg_add for a detailed description of the other fields.
# As in pkg_add, the "new version" and "arch" are optional.
#
########################################

define function pkg_repl = {
    pkg_list = self;
    name = argv[0];
    u_name = '_'+name;
    if (argc > 1 && argv[1] != DEF) {
      version = argv[1];
    } else {
      if (exists(package_default[u_name][0])) {
	version = package_default[u_name][0];
      } else {
        error("no default package version defined for package: "+name);
      };
    };
    if (argc > 2 && argv[2] != DEF) {
      arch = argv[2];
    } else {
      if (exists(package_default[u_name][1])) {
        arch = package_default[u_name][1];
      } else {
        error("no default package architecture defined for package: "+name);
      };
    };
    debug("Replacing package " + name + "new version: "+ version + " arch " + arch);
    e_name = escape(name);
    e_version = escape(version);
    if(argc > 3 && argv[3] != DEF) {
	old_version = argv[3];
        e_old_version = escape(old_version);
        debug ("   old version being replaced: "+old_version);
    } else {
	e_old_version = DEF;
	debug ("   old version not specified, replacing all versions");
    };
    if(argc > 4) {
	flags = argv[4];
    } else {
	flags = DEF;
    };
    if(argc > 5) {
	rep = argv[5];
    } else {
	rep = DEF;
    };
    if(e_old_version != DEF) {
	delete(pkg_list[e_name][e_old_version]);
    } else {
	delete(pkg_list[e_name]);
    };
    pkg_list[e_name][e_version] = nlist("arch",arch);
    if(is_nlist(flags)) {
	pkg_list[e_name][e_version]["flags"] = flags;
    };
    if(rep != DEF) {
        pkg_list[e_name][e_version]["repository"] = rep;
    };
    return(pkg_list);
};


# Replace package in the list ONLY if present
#
# pkg_ronly <name> <new version> <arch> [old version] [flags] [repository]
#
# if old version is not specified, all versions are replaced.
#
# if no version existed in the profile, NO new version is added
#
# flags and repository: see pkg_add.
#
########################################

define function pkg_ronly = {
    pkg_list = self;
    name = argv[0];
    version = argv[1];
    arch = argv[2];
    e_name = escape(name);
    e_version = escape(version);
    if(argc > 3 && argv[3] != DEF) {
	old_version = argv[3];
        e_old_version = escape(old_version);
    } else {
	e_old_version = DEF;
    };
    if(argc > 4) {
	flags = argv[4];
    } else {
	flags = DEF;
    };
    if(argc > 5) {
	rep = argv[5];
    } else {
	rep = DEF;
    };
    if (exists (pkg_list[e_name])) { # already existing?
         debug("Replacing package " + name + "new version: "+ version + " arch " + arch);
         if(e_old_version != DEF) {
	   debug ("   old version being replaced: "+old_version);
  	   delete(pkg_list[e_name][e_old_version]);
	 } else {
	   delete(pkg_list[e_name]);
	   debug ("   old version not specified, replacing all versions");
	 };
	 pkg_list[e_name][e_version] = nlist("arch",arch);
	 if(is_nlist(flags)) {
	   pkg_list[e_name][e_version]["flags"] = flags;
	 };
	 if(rep != DEF) {
	   pkg_list[e_name][e_version]["repository"] = rep;
	 };
    };
    return(pkg_list);
};
