###################################################################
#
# Validation functions definition for the following composed types:
#
#     * software component
#     * /software tree
#     * /system tree
#
# RESPONSIBLE: Rafael Garcia Leiva <angel.leiva@uam.es>
# VERSION: 1.1.7
#
###################################################################

declaration template pro_declaration_structure_validation_functions;

include pro_declaration_functions_general;


#######################################################################
#
# pro_declaration_structure_validation_component
# 
# VERSION: 1.1.7
#
# created by Martin Bock <bock@delta.ft.uam.es>    (2004-06-22)
#
#######################################################################

define function is_component = {
   # general checks
   if( argc != 1 ) error("expecting precisely one argument");
   if( !is_nlist(argv[0]) ) error("expecting nlist");

   # checking active, dispatch flag
   if( !exists(argv[0]["active"]) || !is_boolean(argv[0]["active"]) ) {
      error("missing \"active\" : boolean");
      };
   if( !exists(argv[0]["dispatch"]) || !is_boolean(argv[0]["dispatch"]) ) {
      error("missing \"dispatch\" : boolean");
      };
   # check register change list
   if( exists(argv[0]["register_change"]) ) {
      if( !is_list(argv[0]["register_change"]) ) {
         error("register_change: should be list");
         };
      aaa = 0;
      while( aaa < length(argv[0]["register_change"]) ) {
         if( !exists(to_string(argv[0]["register_change"][aaa])) ) {
            error("register change path does not exist: " +
                  argv[0]["register_change"][aaa]);
            };
         aaa = aaa + 1;
         };
      };
   # check dependencies
   if( exists(argv[0]["dependencies"]) ) {
      if( !is_nlist(argv[0]["dependencies"]) ) {
         error("dependencies: should be nlist");
         };
      components = value("/software/components");
      # pre
      if( exists(argv[0]["dependencies"]["pre"]) ) {
         if( !is_list(argv[0]["dependencies"]["pre"]) ) {
            error("dependencies/pre: should be list");
            };
         pre = argv[0]["dependencies"]["pre"];
         bbb = 0;
         while(bbb < length(pre) ) {
            if( !is_string(pre[bbb]) ) {
               error("entries of dependencies/pre should be stings");
               };
            if( !exists( components[pre[bbb]] ) ) {
               error("pre dependency " + pre[bbb] + "not a component");
               };
            bbb = bbb + 1;
            };
         }; # end pre
      # post
      if( exists(argv[0]["dependencies"]["post"]) ) {
         if( !is_list(argv[0]["dependencies"]["post"]) ) {
            error("dependencies/post: should be list");
            };
         post = argv[0]["dependencies"]["post"];
         bbb = 0;
         while(bbb < length(post) ) {
            if( !is_string(post[bbb]) ) {
               error("entries of dependencies/post should be stings");
               };
            if( !exists( components[post[bbb]] ) ) {
               error("post dependency " + post[bbb] + "not a component");
               };
            bbb = bbb + 1;
            };
         }; # end post
      }; # end check dependencies

   # once we are here there should be no problems
   return(true);
};


#######################################################################
#
# pro_declaration_structure_validation_function_software
# validation function for the /software tree
#
# VERSION: 1.1.7
#
# created by Martin Bock <bock@delta.ft.uam.es> (2004-06-22)
#
#######################################################################

define function is_components = {
   # basic checks
   if( argc != 1 ) error("expecting precisely one argument");
   if( !is_nlist(argv[0]) ) error("expecting nlist");

   # no traverse through the list and check wether all are valid components
   aaa = first(argv[0], component, settings);
   while(aaa) {
      if(!is_component(settings)) error("not of component type: " + component);
      aaa = next(argv[0], component, settings);
      };

   # no problems if we arrive here
   return(true);
};

   
#######################################################################
#
# pro_declaration_structure_validation_function_system
# validation functions related to /system, mainly /system/filesystems
#
# 1.1.7
#
# created by Martin Bock <bock@delta.ft.uam.es> (2004-06-22)
#
#######################################################################

define function is_structure_disk = {
   # first do some general testing
   if ( argc != 2 ) {
      error("usage: is_structure_disk(disk_device_file, nlist:partitions");
      };
   if ( is_resource(argv[0]) ) {
      error("usage: is_structure_disk(disk_device_file, nlist:partitions");
      };
   
   disk_devfile = argv[0];
   partitions   = argv[1];

   if ( length(partitions) == 0 ) {
      debug("Is it really intended to set an empty partition table on " +
      disk_devfile + " ?");
      };
   
   # then traverse the whole tree to gather information and for more checks
   primary_used_size  = 0;
   extended_partition_exists = false;
   ccc = first(partitions, partition_name, partition);
   while(ccc) {
   
      # first check the naming of primary partitions
      partition_devfile = matches(partition_name, '^([hs]d[a-z])(\d+)$');
      if ( length(partition_devfile) != 3) {
         error("no valid primary partition device file: " + partition_name);
         };
      if ( partition_devfile[1] != disk_devfile) {
         error("primary partition " + partition_name + " defined on harddisk " +
         disk_devfile);
         };
      if ( to_long(partition_devfile[2]) > 4) {
         error("cannot be primary partition: " + partition_name);
         };

      # extra sanity checks for primary partitions
      if ( exists(partition["type"]) ) { # if partition has type
         if ( partition["type"] != "extended" ) {
            if ( !exists(partition["mountpoint"])) {
	       error("no mountpoint defined for primary partition " +
               partition_name);
	       };
            if ( exists(partition["logical_partitions"]) ) {
	       error("logical partitions defined on primary partition with " +
               "filesystem: " + partition_name);
	       };
            };
      } else { # if partition has no type
          if ( exists(partition["mountpoint"]) ) {
	    error("mountpoint defined on partition without type " +
               partition_name);
	    };
         if ( exists(partition["mountoptions"]) ) {
            error("mount options defined on partition without type " +
               partition_name);
            };
         if ( exists(partition["freq"]) ) {
            error("dump frequency number defined on partition without type " +
               partition_name);
            };
         if ( exists(partition["pass"]) ) {
            error("fsck pass number defined on partition without type " +
               partition_name);
            };
         if ( exists(partition["logical_partitions"]) ) {
            error("logical partitions defined on partition without type " +
               partition_name);
	    };
         }; # end estra sanity checks for primary partitions
      
      # check of extended partition
      if ( exists(partition["type"]) && partition["type"] == "extended") {
         if( extended_partition_exists ) {
            error("more than one extended partition");
            };

         if ( exists(partition["mountpoint"]) ) {
	    error("mountpoint defined on extended partition " + partition_name);
	    };
         if ( exists(partition["mountoptions"]) ) {
            error("mount options defined on extended partition " +
            partition_name);
            };
         if ( exists(partition["freq"]) ) {
            error("dump frequency number defined on extended partition " +
            partition_name);
            };
         if ( exists(partition["pass"]) ) {
            error("fsck pass number defined on extended partition " +
            partition_name);
            };
         if ( length(partition["logical_partitions"]) == 0 ) {
	    debug("Is it really intended to define no logical partitions on " +
            partition_name + " ?");
	    };

         # check logical partitions  
         aaa = first(partition["logical_partitions"], logical_partition_name,
               logical_partition);
         logical_used_size = 0;
         while(aaa) { 
            # first check the naming of partitions
            logical_partition_devfile = matches(logical_partition_name,
            '^([hs]d[a-z])(\d{1,2})$');
            if ( length(logical_partition_devfile) != 3) {
               error("no valid logical partition device file: " +
               logical_partition_name);
               };
            if ( logical_partition_devfile[1] != disk_devfile) {
               error(logical_partition_name + " defined" + " on " +
               disk_devfile);
               };
            if ( to_long(logical_partition_devfile[2]) < 5 ) {
               error("no valid logical partition name: "
               + logical_partition_name);
               };

            # are the preceding logical partitions defined?
            bbb = 5;
            while(bbb < to_long(logical_partition_devfile[2]) ) {
               if ( !exists(partition["logical_partitions"][disk_devfile +
               to_string(bbb)] ) ) {
                  error("preceding logical partitions of " +
                  logical_partition_name + " not defined");
                  };
               bbb = bbb + 1;
               };
            
            # increment counters
            logical_used_size = logical_used_size +
                                logical_partition["size"];
            aaa = next(partition["logical_partitions"], logical_partition_name,
            logical_partition);
            }; ##### end check of logical partitions

         # check wether logical partitions are not too big
         if (logical_used_size > partition["size"] ) {
            error("logical partitions on " + partition_name + " use more space"
            + " than available");
            };

         extended_partition_exists = true;
         }; ##### end check of extended partition
      
         # are the preceding primary partitions defined?
         ddd = 1;
         while(ddd < to_long(partition_devfile[2]) ) {
            if ( !exists(partitions[disk_devfile + to_string(ddd)] ) ) {
               error("preceding primary partitions of " +
               partition_name + " not defined");
               };
            ddd = ddd + 1;
            };
         
      # increment counters
      primary_used_size = primary_used_size + partition["size"];
      ccc = next(partitions, partition_name, partition);
      }; # now we are trough all primary partitions

   # check that the harddisk has enough space for our primary partitions
   harddisks = value("/hardware/harddisks");
   if( primary_used_size > harddisks[disk_devfile]["capacity"] ) {
      error( disk_devfile + " has not enough space for all partitions");
      };
      
   # once we are here there should be no problems
   return(true);
};

define function is_structure_disks = {
   # basic checks
   if (argc != 1) error("not a valid set of disk partition info");
   disks = argv[0];
   harddisks = value("/hardware/harddisks");
   
   aaa = first(disks, disk, partitions);
   while(aaa) {
      # check wether this disk is also in the hardware tree
      if ( !exists(harddisks[disk]) ) {
         error("cannot find hardware for disk " + disk);
         };
      # check wether partition structure is valid
      if( !is_structure_disk(disk, partitions) ) {
         error("undetermined error with partition info for disk " + disk);
         };
      aaa = next(disks, disk, partitions);
      };

   # once we are here everything should be ok
   return(true);
};

define function is_structure_filesystems = {
   # checks that there are no double appearing mount points
   
   # check cardinality
   if (argc != 1) error("not a valid filesystem info");
   
   fsys = argv[0];
   mounts = undef;
   
   # first check imports
   if( exists(fsys["imports"]) && is_resourcre(fsys["imports"]) ) {
      imports = fsys["imports"];
      aaa = 0;

      while( aaa < length(imports) ) {
         mounts = check_mount(mounts, imports[aaa]);
         aaa = aaa + 1;
         };
      }; # end check imports

   # then check disks
   if( exists(fsys["disks"]) && is_resource(fsys["disks"]) ) {
      disks = fsys["disks"];

      # start traversing through disks tree
      bbb = first(disks, disk, pars);
      while(bbb) {
         # start traversing through disk subtree
         ccc = first(pars, par, parset);
         while(ccc) {

            # check wether there are logical pars, handle them
            if( exists(parset["logical_partitions"]) && 
                  is_nlist(parset["logical_partitions"]) ) {
               # start traversing through logical pars
               ddd = first(parset["logical_partitions"], logical_par,
                     logical_parset);
               while(ddd) {
                  mounts = check_mount(mounts, logical_parset);
                  ddd = next(parset["logical_partitions"], logical_par, 
                        logical_parset);
                  }; # end traversing through logical pars
               } else { # finished handling logical pars

               # now for the primary ones
               mounts = check_mount(mounts, parset)
               }; # finished with one primary par
            ccc = next(pars, par, parset);
            }; # finished with par info of one disk

         bbb = next(disks, disk, pars);
         }; # finished with all disks
      }; # end check disks

   # check wether there is a mount for /
   if( !exists(mounts[escape("/")]) ) error("no root filesystem");

   # once we are here there should be no problems
   return(true);
};

define function check_mount = {
   # adds the mountpoint entry of a given resource to the mount table, if it
   # does not exist there so far, else drops an error
   
   # general testing
   if( argc != 2 || !is_resource(argv[1]) ) {
      # mount table may be empty
      error("expecting two resources");
      };
   
   mounts = argv[0];
   partition = argv[1];
   
   # we test only if partition not swap and has a mounpoint
   if( !exists(partition["mountpoint"]) ) return(mounts);
   if( match(partition["mountpoint"], '^(swap|none)$') ) return(mounts);

   # if mountpoint not already defined append to mounts
   if( !exists(mounts[ escape(partition["mountpoint"]) ]) ) {
      mounts[ escape(partition["mountpoint"]) ] = 1;
      return(mounts);
      };

   # so such a mountpoint exists already
   error("multiply appearing mountpoint " + partition["mountpoint"]);
};
