###################################################################
#
# Structure definition for the following structures:
#
#     * annotation
#     * ram
#     * harddisk
#     * cpu
#     * nic
#     * cards
#     * hardware
#     * interface
#     * network
#     * edg
#     * software
#     * system
#     * component
#     * profile
#
# RESPONSIBLE: Rafael Garcia Leiva <angel.leiva@uam.es>
# VERSION: 1.1.7
#
###################################################################

declaration template pro_declaration_structures;

include pro_declaration_types;
include pro_declaration_structure_validation_functions;

#######################################################################
#
# pro_declaration_structure_annotation
# optional hardware specific information
#
# VERSION: 1.1.7
#
# created by Martin Bock <bock@delta.ft.uam.es>
#                                         (2004-06-21)
#
#######################################################################

define type structure_annotation = {
   "name"         ? string descro "product name"
   "type"         ? string descro "product type"
   "model"        ? string descro "product model"
   "manufacturer" ? string descro "manufacturer name"
   "vendor"       ? string descro "vendor name"
   "version"      ? string descro "product version"
   "chipset"      ? string descro "product chipset"
   "serialnumber" ? string descro "product serialnumber"
   "arch"         ? string descro "product architecture"
                     # i386, amd64, m68k, ...
   "bus"          ? string descro "bus of peripheral"
   "clock"        ? long   descro "clock of peripheral"
   "lang"         ? string descro "language of the product"
   "power"        ? long   descro "power in watts"
   "location"     ? string descro "location of the hardware"
   # ...
   # please do add your stuff here and enhance it by whatever you need!
};


############################################################
#
# type definition pro_declaration_structure_ram
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# include structure_annotation            (2004-06-24)
#
############################################################

define type structure_ram = {
  include structure_annotation
  "name" ? string
  "size" : long descro "Size of module in MB"
  # usually given in N*MB or N*GB or (possibly) N*TB as
  # defined in pro_declaration_units.tpl
  "data_rate" ? string
};


############################################################
#
# type definition pro_declaration_structure_harddisk
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# added annotation structure                 (2004-06-24)
#
############################################################

define type structure_harddisk = {
  include structure_annotation
  "capacity"  : long descro "Capacity in mb"
  # usually given in N*mb or N*gb or N*tb as defined in
  # pro_declaration_units.tpl
  "interface" : string
};


############################################################
#
# type definition pro_declaration_structure_cpu
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# include structure_annotation            (2004-06-24)
#
############################################################

define type structure_cpu = {
  include structure_annotation
  "speed"  : long descro "CPU clock speed in MHz"
  # usually given in N * MHz or N * GHz as defined in
  # pro_declaration_functions.tpl
};


############################################################
#
# type definition pro_declaration_structure_nic
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# include annotation structure               (2004-06-24)
# conform to pan user conventions            (2004-07-23)
#
############################################################

define type structure_nic = {
  include structure_annotation
  "hwaddr"       : type_hwaddr
  "driver"	 ? string
  "media"        ? string
  "name"         ? string
};


############################################################
#
# type definition pro_declaration_structure_cards
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integraton with quattor build tools        (2004-06-24)
#
############################################################

define type structure_cards = {
    "nic"      : structure_nic{}
  ## for example
  # "interface_xyz" : structure_interface_xyz[]
  ## for some USB or IEEE1394 interface that needs to be configured
  ## mind also to define a type for this (like 
  ## pro_declaration_structure_interface_xyz) and include it before
  ## referencing
};


############################################################
#
# type definition pro_declaration_structure_hardware
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integrate with quattor build tools      (2004-06-24)
#
############################################################

define type structure_hardware = {
  include structure_annotation
  "cpu"          ? structure_cpu[]
  "harddisks"    ? structure_harddisk{}
  "ram"          ? structure_ram[]
  "cards"        ? structure_cards
};


############################################################
#
# type definition pro_declaration_structure_interface
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integrate with quattor build tools      (2004-06-25)
#
############################################################

define type structure_interface = {
  "ip"      ? type_ip
  "gateway" : type_ip
  "netmask" : type_ip
  "broadcast" : type_ip
  "driver"  ? string
};


############################################################
#
# type definition pro_declaration_structure_network
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integrate with quattor build tools      (2004-06-25)
# remove timeservers                      (2004-07-26)
#
############################################################

define type structure_network = {
    "domainname" : type_fqdn
    "hostname"   : type_shorthostname
    "interfaces" : structure_interface{}
    "nameserver" : type_ip[] 
};


############################################################
#
# type definition for EDG (legacy support for DataGrid)
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integrate with quattor build tools         (2004-06-24)
#
############################################################

define type structure_edg_config = {
	"EDG_LOCATION" : string
	"EDG_LOCATION_VAR" ? string
	"EDG_TMP" : string 
	"CERTDIR" ? string
	"X509_USER_CERT" ? string
	"X509_USER_KEY" ? string
	"GRIDMAP" ? string
	"GRIDMAPDIR" ? string
        "EDG_WL_BKSERVERD_ADDOPTS" ? string
        "EDG_WL_RGMA_FILE" ? string
        "EDG_WL_RGMA_SOCK" ? string

};

define type structure_edg = {
	"config" ? structure_edg_config
};

############################################################
#
# SOFTWARE: Type definitions
#
# Author: Andrey Kiryanov <Andrey.Kiryanov@cern.ch>
#
# (modif by GC 17/3/03)
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# TPL13, bug 3661: added is_components    (2004-06-22)
#
############################################################

define type SOFTWARE_PACKAGE_FLAGS = {
    "reboot" ? boolean descro "Needs reboot after install ?"
    "offprod" ? boolean descro "Don't install if node in production ?"
    "mandatory" ? boolean descro "Mandatory package ?"
    "unwanted" ? boolean descro "Unwanted package ?"
};

define type SOFTWARE_PACKAGE = {
    "arch" : string descro "Package architecture"
    "repository" ? string descro "Repository name" with
named_item_exists(self,"/software/repositories")
    "components" ? string[] descro "Depending components"
    "flags" ?  SOFTWARE_PACKAGE_FLAGS
};

define type SOFTWARE_REPOSITORY_PACKAGE = {
    "name" : string  descro "Package name"
    "version" : string  descro "Package version"
    "arch" : string  descro "Package architecture"
};

define type SOFTWARE_REPOSITORY_PROTOCOL = {
    "name" : string  descro "Protocol name"
    "url" : string  descro "URL for the given protocol"
};

define type SOFTWARE_REPOSITORY = {
    "name" : string  descro "Repository name"
    "owner" ? string  descro "Contact person (email)"
    "protocols" : SOFTWARE_REPOSITORY_PROTOCOL []
    "contents" ? SOFTWARE_REPOSITORY_PACKAGE {} {}
};

define type SOFTWARE = {
    "components" ? nlist with is_components(self) descro "NCM components"
    "repositories" ? SOFTWARE_REPOSITORY []
    "packages" ? SOFTWARE_PACKAGE {} {}
};


############################################################
#
# type definition pro_declaration_structure_system
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes:
# task 732: /system/filesystems		(2004-06-12) (Martin Bock)
# bugs 4493: mount point		(2004-09-06) (R. Garcia Leiva)
#
############################################################

define type structure_logical_partition = {
   "size"         : long
   "type"         ? string with match(self,
                    '^(ext3|ext2|reiserfs|xfs|jfs|swap)$')
                    # more possibilities?
   "label"        ? string
   "options"      ? string
   "preserve"     ? boolean         # do not reformat partition
   "mkprog"       ? string          # mkfs program path    <optional>
   "mkopts"       ? string          # mkfs options         <optional>
   "ksopt"        ? string          # kickstart options
   "quota"        ? long            # quota percentage
   # and also the mount information
   # use a generic validation code for mount points:
   "mountpoint"   ? string with match(self,
                    '^(/[a-zA-Z\_\d/]*|none|swap)$')
   # alternatively, you can use a more restrictive approach
   # by providing a list of valid mount points:
   # "mountpoint"   ? string with match(self,
   # '^(/|/usr|/usr/lib|/usr/bin|/usr/local|/boot|/home|/var|/tmp|/opt|/var/www|none|swap)$')
   "mountoptions" ? string          # mount options
   "freq"         ? long            # dump frequency
   "pass"         ? long            # fsck pass number
};

define type structure_partition = {
   # more sanity checks in is_structure_disk()
  "size"          : long
  "type"          ? string with match(self,
                    'extended|ext3|ext2|reiserfs|xfs|jfs|swap')
                    # more possibilities?
  "label"         ? string
  "options"       ? string
  "preserve"      ? boolean         # do not reformat partition
  "mkprog"        ? string          # mkfs program path    <optional>
  "mkopts"        ? string          # mkfs options         <optional>
  "ksopt"         ? string          # kickstart options
  "quota"         ? long            # quota percentage
  "logical_partitions" ? structure_logical_partition{}
   # and also the mount information
   "mountpoint"   ? string with match(self,
                   
'^(/|/usr|/usr/lib|/usr/bin|/usr/local|/boot|/home|/var|/tmp|/opt|/var/www|none|swap)$')
                    # more real-world used mountpoints
   "mountoptions" ? string          # mount options
   "freq"         ? long            # dump frequency
   "pass"         ? long            # fsck pass number
};

define type structure_disk = structure_partition{} with( length(self) < 5 );

define type structure_import = {
  "source"     : string # with isURI(self)
  "mountpoint" : string with match(self,
                
'^(/|/usr|/usr/lib|/usr/bin|/usr/local|/boot|/home|/var|/tmp|/opt|/var/www)$')
                 # more real-world used mountpoints?
  "options"    ? string
};

define type structure_export = {
  "localpath" : string
  "options"   ? string
};

define type structure_filesystems = {
  "disks"  ? structure_disk{} with is_structure_disks(self)
  "imports" ? structure_import[]
  "exports" ? structure_export[]
} with is_structure_filesystems(self);

define type structure_cluster = {
  "name" : string
  "type" ? string
};

define type structure_kernel = {
  "version": string descro "kernel version to use (eg. 2.4.18-27.7.x.cernsmp)"
};

define type structure_oldname = {
  "name" : string
  "date" : string
};

define type structure_system = {
  "network"     : structure_network
  "cluster"     ? structure_cluster
  "kernel"      ? structure_kernel
  "filesystems" ? structure_filesystems
  "state"       ? string with match (self,
'production|standby|test|development|onloan') descro
"'production|out-of-production|test|development|onloan"
  "siterelease" ? string
  "rootmail"    ? type_email
  "oldnames"    ? structure_oldname[]
  "edg"         ? structure_edg
# CAL 2004/12/04 Added this to make consistent with LCG usage.
  "lcg"         ? structure_lcg
# CAL 2004/12/04 Added VO information to structure.
  "vo"          ? structure_vo{}
};


############################################################
#
# type definition pro_declaration_structure_component
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# naming convention                       (2004-04-26)
# according task #708                     (2004-06-21)
#
############################################################

define type structure_component_dependency = {
	"pre"          ? string[]
	"post"         ? string[]
};

define type structure_component = {
	"active"	: boolean descro "component active?"
        "dispatch"      : boolean descro "run with cdispd?"
	"register_change" ? string[]
        "dependencies"  ? structure_component_dependency
};


############################################################
#
# definition of type pro_declaration_structure_profile;
#
# RESPONSIBLE: German Cancio <German.Cancio@cern.ch>
#
# VERSION: 1.1.7
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integrate with quattor build tools      (2004-06-24)
#
############################################################

define type structure_profile = {
  "hardware" : structure_hardware
  "software" ? SOFTWARE 
  "system"   : structure_system
};
