###################################################################
#
# Data types definition for the following basic types:
#
#     * date (deprecated)
#     * ansdate
#     * isodate
#     * hwaddr
#     * ip (ipv4 and ipv6)
#     * fqdn
#     * hostname
#     * shorthostname
#     * port
#     * hostport
#     * URI
#     * absuoluteURI
#     * hostURI
#     * email
#
# RESPONSIBLE: Rafael A. Garcia Leiva <angel.leiva@uam.es>
# VERSION: 1.1.7
#
###################################################################

declaration template pro_declaration_types;

include pro_declaration_type_validation_functions;

############################################################
#
# Type definition for Date and Time
# XXX DEPRECATED XXX
# use well standardized isodate instead
#
# (supersedes type definition pro_declaration_type_date.tpl)
#
# Author: Piotr Poznanski <Piotr.Poznanski@cern.ch>
# --created vicariously by Bill Tomlin
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integrate with quattor build tools      (2004-06-25)
#
# RESPONSIBLE: Rafael A. Garcia Leiva <angel.leiva@uam.es>
#
############################################################

define type type_date = string with {
  is_date(self);
};

############################################################
#
# This type implements a date/time format consistent with 
# the that used in LDAP.
#
# (supersedes type definition pro_declaration_type_asndate)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type type_asndate = string with {
  is_asndate(self);
};

############################################################
#
# This type implements a date/time format consistent with 
# the W3C use of the datetime format.  See the document:
#
# http://www.w3.org/TR/NOTE-datetime
# 
# This is a subset of the ISO8601 specification. 
#
# (supersedes type definition pro_declaration_type_isodate)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type isodate = string with {
  is_isodate(self);
};

############################################################
#
# Hardware address (MAC) type.
#
# (supersedes type definition pro_declaration_type_hwaddr)
#
# RESPONSIBLE       : Charles Loomis <charles.loomis@cern.ch>
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integration with quattor build tools    (2004-06-25)
#
############################################################

define type type_hwaddr = string with {
  is_hwaddr(self);
};

############################################################
#
# This address represents a numerical IPv4 address.
#
# (supersedes type definition pro_declaration_type_ipv4)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type ipv4 = string with {
  is_ipv4(self);
};

############################################################
#
# This address represents a numerical IPv6 address.
#
# (supersedes type definition pro_declaration_type_ipv6)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type ipv6 = string with {
  is_ipv6(self);
};

############################################################
#
# This type represents either an IPv4 or IPv6 numerical
# address. 
#
# (supersedes type definition pro_declaration_type_ip)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type type_ip = string with {
  (is_ipv4(self) || is_ipv6(self));
};

############################################################
#
# This represents a fully-qualified domain name. 
#
# (supersedes type definition pro_declaration_type_fqdn)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type type_fqdn = string with {
  is_fqdn(self);
};

############################################################
#
# This represents a hostname--either a fully-qualified
# domain name (fqdn) or an IP address (ip).  The IP address
# can be in either IPv4 or IPv6 format. 
#
# (supersedes type definition pro_declaration_type_hostname)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type type_hostname = string with {
  is_hostname(self);
};

############################################################
#
# This represents a short hostname
# in contrary to a fully-qualified
# domain name.
#
# (supersedes type definition pro_declaration_type_hostname)
#
# created by Martin Bock <bock@delta.ft.uam.es> (2004-04-27)
#
# RESPONSIBLE: Rafael A. Garcia Leiva <angel.leiva@uam.es>
#
############################################################

define type type_shorthostname = string with {
  (is_shorthostname(self));
};

############################################################
#
# This represents a port.
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type type_port = long with {
  is_port(self);
};

############################################################
#
# This represents a host and port of the form hostname:port.
#
# (supersedes type definition pro_declaration_type_hostport)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type type_hostport = string with {
  is_hostport(self);
};

############################################################
#
# Type URI according to RFC2396.
#
# (supersedes type definition pro_declaration_type_URI)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type URI = string with {
  is_URI(self);
};

############################################################
#
# Type absolute URI according to RFC2396. This is 
# just a valid URI with the scheme explicitly included.
#
# (supersedes type definition pro_declaration_type_absoluteURI)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type absoluteURI = string with {
  is_absoluteURI(self);
};

############################################################
#
# Type host URI according to RFC2396. A host-based URI is
# an absolute URI with a non-empty host field. 
#
# (supersedes type definition pro_declaration_type_hostURI)
#
# RESPONSIBLE: Charles Loomis <charles.loomis@cern.ch>
#
############################################################

define type type_hostURI = string with {
  is_hostURI(self);
};

############################################################
#
# Type definition for emails.
#
# (supersedes type definition pro_declaration_type_email)
#
# Author: Piotr Poznanski <Piotr.Poznanski@cern.ch>
# --created vicariously by Bill Tomlin
#
# changes by Martin Bock <bock@delta.ft.uam.es>
# integrate with quattor build tools      (2004-06-25)
#
# RESPONSIBLE: Rafael A. Garcia Leiva <angel.leiva@uam.es>
#
############################################################

define type type_email = string with {
  is_email(self);
};
